local WorldQuestGroupFinderAddon = CreateFrame("Frame", "WorldQuestGroupFinderAddon", UIParent)
local _, L = ...;

WorldQuestGroupFinder = {}
WorldQuestGroupFinderConfig = {}

local DefaultConfig = {}
local RegisteredEvents = {}
WorldQuestGroupFinderAddon:SetScript("OnEvent", function (self, event, ...) if (RegisteredEvents[event]) then return RegisteredEvents[event](self, event, ...) end end)

local currentWQ = nil
local tempWQ = nil

local activityIDs = {
	["419"] = "Aszuna",
	["420"] = "Val'sharah", 
	["421"] = "Highmountain",
	["422"] = "Stormheim", 
	["423"] = "Suramar"
}

local pendingApplications = {}
local blacklistedLeaders = {}
local seenWorldQuests = {}

function RegisteredEvents:ADDON_LOADED(event, addon, ...)
	if (addon == "WorldQuestGroupFinder") then
		SLASH_WQGF1 = '/wqgf'
		SlashCmdList["WQGF"] = function (msg, editbox)
			WorldQuestGroupFinder.handleCMD(msg, editbox)	
		end
		setmetatable(WorldQuestGroupFinderConfig, {__index = DefaultConfig})
		print("|c00bfffffWorld Quest Group Finder v0.3 BETA. "..L["Click on a world quest in the objective tracking window to search for a group."])
	end
end

function RegisteredEvents:PLAYER_LOGIN(event)
	WorldQuestGroupFinder.SetHooks()
	WorldQuestGroupFinderConf.CreateConfigMenu()
end

function RegisteredEvents:QUEST_TURNED_IN(event, questID, experience, money)
	if (currentWQ ~= nil and questID == currentWQ) then
		if (WorldQuestGroupFinderConf.GetConfigValue("notifyParty") and IsInGroup()) then
			WorldQuestGroupFinder.SendWQCompletionPartyNotification(questID)
		end
		if (WorldQuestGroupFinderConf.GetConfigValue("askToLeave")) then
			WorldQuestGroupFinder.ShowLeavePrompt()
		end
		currentWQ = nil
	end
end

function RegisteredEvents:LFG_LIST_APPLICATION_STATUS_UPDATED(event, applicationID, status)
	local _,_,name,description,_,ilvl,_,_,_,_,_,_,author,members = C_LFGList.GetSearchResultInfo(applicationID)
	if (status == "applied") then
		pendingApplications[applicationID] = tempWQ
	end
	if (status == "declined" or status == "failed" or status == "invitedeclined" or status == "timedout") then
		blacklistedLeaders[author] = true
		-- If all applications have been declined, restart process. A new group will be created if nothing else is found.
		if (C_LFGList.GetNumApplications() - 1 == 0) then
			WorldQuestGroupFinder.InitWQGroup(pendingApplications[applicationID])
		end
	end
	if (status == "cancelled") then
		-- placeholder
	end
	if (status == "inviteaccepted") then
		WorldQuestGroupFinder.prefixedPrint(string.format(L["You have joined |c00bfffff%s|c00ffffff's group for |c00bfffff%s|c00ffffff. Have fun!"], author, WorldQuestGroupFinder.GetQuestInfo(pendingApplications[applicationID])))
		currentWQ = pendingApplications[applicationID]
		local pendingApplications = {}
		local blacklistedLeaders = {}
	end
end

function RegisteredEvents:GROUP_ROSTER_UPDATE(event)
	-- Leaving the group. TRIGGERED TWICE
	if (IsInGroup() == false) then
		currentWQ = nil
	end
end

for k, v in pairs(RegisteredEvents) do
	WorldQuestGroupFinderAddon:RegisterEvent(k)
end

function WorldQuestGroupFinder.GetQuestInfo (questID)
	local tagID, tagName, worldQuestType, rarity, elite, tradeskillLineIndex = GetQuestTagInfo (questID)
	local title, factionID = C_TaskQuest.GetQuestInfoByQuestID (questID)
	return title, factionID, tagID, tagName, worldQuestType, rarity, elite, tradeskillLineIndex
end

function WorldQuestGroupFinder.SetHooks()	
	local bkp_BonusObjectiveTracker_OnBlockClick = BonusObjectiveTracker_OnBlockClick
	function BonusObjectiveTracker_OnBlockClick(self, button)	
		if (button == "LeftButton") then
			tempWQ = self.TrackedQuest.questID
			if (currentWQ ~= nil and tempWQ ~= currentWQ) then
				StaticPopup_Show ("NEW_WORLD_QUEST_PROMPT")
			else 
				-- No current WQ. 
				if (currentWQ == nil) then
					WorldQuestGroupFinder.InitWQGroup(tempWQ)
				else
					WorldQuestGroupFinder.prefixedPrint(L["You already are in a group for this world quest."])
				end
			end
		end
		bkp_BonusObjectiveTracker_OnBlockClick(self, button)
	end
	
	local bkp_BonusObjectiveTracker_OnBlockAnimInFinished = BonusObjectiveTracker_OnBlockAnimInFinished
	function BonusObjectiveTracker_OnBlockAnimInFinished(self) 
		if (WorldQuestGroupFinderConf.GetConfigValue("askZoning")) then
			local block = self:GetParent();
			local title, factionID, tagID, tagName, worldQuestType, rarity, elite, tradeskillLineIndex = WorldQuestGroupFinder.GetQuestInfo(block.id)
			-- Ignore pet battle and dungeon quests
			if (worldQuestType ~= LE_QUEST_TAG_TYPE_PET_BATTLE and worldQuestType ~= LE_QUEST_TAG_TYPE_DUNGEON) then
				tempWQ = block.id
				if (not IsInGroup() and WorldQuestGroupFinderConf.GetConfigValue("askZoningAuto")) then
					WorldQuestGroupFinder.InitWQGroup(tempWQ)
				else
					-- Check if the world quest zone has already been entered during this session
					if not seenWorldQuests[block.id] then
						-- No current WQ. 
						if (currentWQ == nil) then
							seenWorldQuests[block.id] = true
							StaticPopup_Show ("WORLD_QUEST_ENTERED_PROMPT", title)
						-- Already doing another WQ
						elseif (currentWQ ~= nil and tempWQ ~= currentWQ) then
							if not WorldQuestGroupFinderConf.GetConfigValue("askZoningBusy") then
								seenWorldQuests[block.id] = true
								StaticPopup_Show ("WORLD_QUEST_ENTERED_SWITCH_PROMPT", title)
							end
						end
					end
				end
			end
		end
		bkp_BonusObjectiveTracker_OnBlockAnimInFinished(self) 
	end
	
	local bkp_BonusObjectiveTracker_TrackWorldQuest = BonusObjectiveTracker_TrackWorldQuest
	function BonusObjectiveTracker_TrackWorldQuest(questID, hardWatch)
		--WorldQuestGroupFinder.AttachButtonToWorldQuest()
		bkp_BonusObjectiveTracker_TrackWorldQuest(questID, hardWatch)
	end
end

function WorldQuestGroupFinder.InitWQGroup(wqID) 
	local title, factionID, tagID, tagName, worldQuestType, rarity, elite, tradeskillLineIndex = WorldQuestGroupFinder.GetQuestInfo(wqID)
	local foundZone = false
	local foundGroup = false
	local currentZone = nil
	-- Ignore pet battle and dungeon quests
	if (worldQuestType == LE_QUEST_TAG_TYPE_PET_BATTLE or worldQuestType == LE_QUEST_TAG_TYPE_DUNGEON) then
		WorldQuestGroupFinder.prefixedPrint(L["This type of world quest is not allowed."])
		return true
	end
	-- Finding current zone activity ID
	for k,v in pairs( activityIDs ) do
		if (C_LFGList.GetActivityInfoExpensive(k)) then
			currentZone = k
			foundZone = true
		end
	end
	if (foundZone) then
		-- Clear existing applications
		if (C_LFGList.GetNumApplications() > 0) then
			for k,v in pairs( C_LFGList.GetApplications() ) do
				C_LFGList.CancelApplication(v)
			end
		end
		WorldQuestGroupFinder.prefixedPrint(string.format(L["Searching for a group for world quest |c00bfffff%s|c00ffffff..."], title))
		-- Search for existing groups
		C_LFGList.ClearSearchResults()
		C_LFGList.Search(1, "#WQ:"..wqID.."#", 0, 4, C_LFGList.GetLanguageSearchFilter());
		-- Delay to let the search finish
		C_Timer.After(2, function()
			local applicationsCount = 0
			local blacklistedApplicationsCount = 0
			local searchCount, searchResults = C_LFGList.GetSearchResults()
			local currentPlayers = 1;
			if (IsInGroup()) then
				currentPlayers = GetNumGroupMembers()
			end
			for k, v in pairs( searchResults ) do
				local _,_,name,description,_,ilvl,_,_,_,_,_,_,author,members = C_LFGList.GetSearchResultInfo(v)
				-- Double check for description content, required ilvl and group members
				if (string.find(description, "#WQ:"..wqID.."#") ~= nil and GetAverageItemLevel() > ilvl and members + currentPlayers <= 5) then
					if (blacklistedLeaders[author] == true) then
						blacklistedApplicationsCount = blacklistedApplicationsCount + 1
					else 
						foundGroup = true;
						local canBeTank, canBeHealer, canBeDamager = UnitGetAvailableRoles("player")
						C_LFGList.ApplyToGroup(v, "WorldQuestGroupFinder User", canBeTank, canBeHealer, canBeDamager)
						applicationsCount = applicationsCount + 1
					end
				end
			end
			C_LFGList.ClearSearchResults()
			-- No group found, creating a new one
			if (foundGroup == false) then
				if (blacklistedApplicationsCount > 0) then
					WorldQuestGroupFinder.prefixedPrint(string.format(L["You have not been applied to %d group(s) because their leader was blacklisted. You can use |c00bfffff/wqgf unbl |c00ffffffto clear the blacklist."], blacklistedApplicationsCount))
				end
				local shortTitle = strsub(title, 0, 31)
				C_LFGList.CreateListing(currentZone, shortTitle, 0, 0, "", "Doing the world quest \""..title.."\" in "..activityIDs[currentZone]..". Automatically created by World Quest Group Finder. #WQ:"..wqID.."#", WorldQuestGroupFinderConf.GetConfigValue("autoinvite"))
				WorldQuestGroupFinder.prefixedPrint(string.format(L["A new group finder entry has been created for |c00bfffff%s|c00ffffff in |c00bfffff%s|c00ffffff."], title, activityIDs[currentZone]))
				currentWQ = wqID
				tempWQ = nil
			else 
				WorldQuestGroupFinder.prefixedPrint(string.format(L["You have been applied to |c00bfffff%d|c00ffffff group(s) for the world quest |c00bfffff%s|c00ffffff."], applicationsCount, title))
			end
		end)
	else
		WorldQuestGroupFinder.prefixedPrint(L["You are not in the right location for this world quest."])
	end
end

StaticPopupDialogs["WORLD_QUEST_FINISHED_LEADER_PROMPT"] = {
	text = L["You have completed the world quest. Would you like to leave the group or unlist it from the Group Finder?"],
	button1 = L["Leave"],
	button2 = L["Stay"],
	button3 = L["Unlist"],
	OnAccept = function()
		LeaveParty()
	end,
	OnAlt = function()
		C_LFGList.RemoveListing()
	end,
	timeout = 0,
	whileDead = true,
	hideOnEscape = true,
	preferredIndex = 3,  
}

StaticPopupDialogs["WORLD_QUEST_FINISHED_PROMPT"] = {
	text = L["You have completed the world quest. Would you like to leave the group?"],
	button1 = "Leave",
	button2 = "Stay",  
	OnAccept = function()
		LeaveParty()
	end,
	timeout = 0,
	whileDead = true,
	hideOnEscape = true,
	preferredIndex = 3,  
}

StaticPopupDialogs["NEW_WORLD_QUEST_PROMPT"] = {
	text = L["You are currently grouped for another world quest.\nAre you sure to want to start another one?"],
	button1 = "Yes",
	button2 = "Cancel",  
	OnAccept = function()
		LeaveParty()
		C_Timer.After(1, function()
			WorldQuestGroupFinder.InitWQGroup(tempWQ)
		end)
		tempWQ = nil
	end,
	OnCancel = function() 
		tempWQ = nil
	end,
	timeout = 0,
	whileDead = true,
	hideOnEscape = true,
	enterClicksFirstButton = true,
	preferredIndex = 3,  
}

StaticPopupDialogs["WORLD_QUEST_ENTERED_PROMPT"] = {
	text = L["You have entered a new world quest area.\nWould you like to find a group for \"%s\"?"],
	button1 = "Yes",
	button2 = "No",  
	OnAccept = function()
		WorldQuestGroupFinder.InitWQGroup(tempWQ)
		tempWQ = nil
	end,
	OnCancel = function() 
		tempWQ = nil
	end,
	timeout = 20,
	whileDead = false,
	hideOnEscape = true,
	preferredIndex = 3,  
}

StaticPopupDialogs["WORLD_QUEST_ENTERED_SWITCH_PROMPT"] = {
	text = L["You have entered a new world quest area, but are currently grouped for another world quest.\nWould you like to leave your current group and find another for \"%s\"?"],
	button1 = "Yes",
	button2 = "No",  
	OnAccept = function()
		LeaveParty()
		C_Timer.After(1, function()
			WorldQuestGroupFinder.InitWQGroup(tempWQ)
			tempWQ = nil
		end)
	end,
	OnCancel = function() 
		tempWQ = nil
	end,
	timeout = 20,
	whileDead = false,
	hideOnEscape = true,
	preferredIndex = 3,  
}

function WorldQuestGroupFinder.SendWQCompletionPartyNotification (questID)
	local title, factionID, tagID, tagName, worldQuestType, rarity, elite, tradeskillLineIndex = WorldQuestGroupFinder.GetQuestInfo(questID)
	SendChatMessage("[WQGF] World quest \""..title.."\" completed! "..L[":)"], "PARTY", "", "");
end

function WorldQuestGroupFinder.ShowLeavePrompt()
	if (UnitIsGroupLeader("player")) then
		StaticPopup_Show ("WORLD_QUEST_FINISHED_LEADER_PROMPT")
	else 
		StaticPopup_Show ("WORLD_QUEST_FINISHED_PROMPT")
	end
end

function WorldQuestGroupFinder.handleCMD(msg, editbox)
	if (string.lower(msg) == "config") then
		InterfaceOptionsFrame_OpenToCategory("WorldQuestGroupFinder")
	elseif (string.lower(msg) == "debug") then
		if (currentWQ ~= nil) then
			WorldQuestGroupFinder.prefixedPrint(string.format(L["Current world quest ID is |c00bfffff%s|c00ffffff."], currentWQ))
		else 
			WorldQuestGroupFinder.prefixedPrint(L["No current world quest."])
		end
		print(L["World quest zones entered this session :"])
		WorldQuestGroupFinder.dumpTable(seenWorldQuests)
	elseif (string.lower(msg) == "showconfig") then
		WorldQuestGroupFinder.dumpTable(WorldQuestGroupFinderConfig)
	elseif (string.lower(msg) == "unbl") then
		blacklistedLeaders = {}
		WorldQuestGroupFinder.prefixedPrint(L["The leaders blacklist has been cleared."])
	else
		WorldQuestGroupFinder.help()
	end
end

function WorldQuestGroupFinder.help()
	print(L["|c00bfffffSlash commands (/wqgf):"])
	print(L["|c00bfffff /wqgf config : Open addon configuration"])
	print(L["|c00bfffff /wqgf unbl : Clear the leaders blacklist"])
end

function WorldQuestGroupFinder.prefixedPrint(text)
	print("|c00bfffff[WQGF]|c00ffffff "..text)
end 

function WorldQuestGroupFinder.dumpTable(t)
	if type(t) == "table" then
		for k, v in pairs( t ) do
			print(k, v)
		end
	else 
		print(t)
	end
end